import sys
import os
import array
import pickle
import glob

import pygame
from pygame.locals import *

from globals import *
import helper
import screen
import tiles

maps = []
newmap = None

def load_maps():
	list = glob.glob(os.path.join('data', 'map*.map'))
	list.sort()
	for fn in list:
		f = open(fn)
		maps.append(pickle.load(f))
	
def save_maps():
	base = os.path.join('data', 'map')
	
	for i in range(len(maps)):
		if i < 10:
			num = '0' + str(i)
		else:
			num = str(i)
		f = open(base + num + '.map', 'w')
		pickle.dump(maps[i], f)

def new_map(size=(100,60), color= (60, 55, 50)):
	global newmap
	newmap = Map(size, color)
	return newmap

def save_new_map():
	global newmap
	newmap.num = len(maps)
	maps.append(newmap)
	save_maps()
	new_map()
	return len(maps) - 1

odd_door = None

class Map:
	
	def __init__(self, size, bg):
		self.num = None
		self.data = array.array('i')
		self.size = size
		self.bg = bg
		for i in range(size[0] * size[1]):
			self.data.append(0)
		self.doors = []
		self.spawns = []
	
	def set_tile(self, loc, tile_num):
		self.data[(loc[1] % self.size[1]) * self.size[0] + (loc[0] % self.size[0])] = tile_num
		#self.data[loc[1] * self.size[0] + loc[0]] 
	
	def resize(self, newsize):
		pass

	def tile(self, loc):
		return self.data[(loc[1] % self.size[1]) * self.size[0] + (loc[0] % self.size[0])]
	
	def obstructed_at(self, loc):
		return self.tile((loc[0] / TILEX, loc[1] / TILEY)) > 19
	
	def obstructed_in(self, area):
		for x in range(area.left / TILEX, ((area.right - 1)  / TILEX) + 1):
			for y in range(area.top / TILEY, ((area.bottom - 1) / TILEY) + 1):
				if self.tile((x, y)) > 19:
					return True
		return False
	
	def door_at(self, location):
		# correct for player sprite size and tile size
		loc = ((location[0] + 8) / TILEX, (location[1] + 24) / TILEY)
		maploc = (loc[1] % self.size[1]) * self.size[0] + (loc[0] % self.size[0])
		for d in self.doors:
			dloc = (d.location[1] % self.size[1]) * self.size[0] + (d.location[0] % self.size[0])
			if dloc == maploc:
				return d
		return None
	
	def add_spawn(self, loc, type):
		if self.num == None:
			return 
		
		location = (loc[1] % self.size[1]) * self.size[0] + (loc[0] % self.size[0])
		
		# remove if already one
		for s in self.spawns:
			if s.location == location:
				self.spawns.remove(s)
				return
		
		new_spawn = SpawnPoint(loc, type)
		self.spawns.append(new_spawn)
	
	def add_door(self, loc):
		global odd_door
		
		location = (loc[1] % self.size[1]) * self.size[0] + (loc[0] % self.size[0])
		
		print location
		
		# no worky on new map
		if self.num == None:
			return 
		
		# remove if already one
		for door in self.doors:
			print door.maploc()
			if door.maploc() == location:
				other = door.target
				if other != None:
					for d in maps[other.mapnum].doors:
						if d.target.maploc() == door.maploc():
							maps[other.mapnum].doors.remove(d)
				self.doors.remove(door)
				if odd_door == door:
					odd_door = None
				return
		
		if odd_door == None:
			new_door = Door(loc, self.num)
			self.doors.append(new_door)
			odd_door = new_door
		else:
			new_door = Door(loc, self.num)
			new_door.target = odd_door
			odd_door.target = new_door
			self.doors.append(new_door)
			odd_door = None	

class SpawnPoint:
	def __init__(self, location, type):
		self.loaded = True
		self.enemy_class = type
		self.location = location
		self.enemy = None
	
	def check_far(self, plr):
		if helper.is_far_tp(self.location, plr.location):
			if not self.loaded:
				self.loaded = True
		else:
			if self.loaded:
				self.loaded = False
				self.enemy = self.enemy_class()
				self.enemy.location = (self.location[0] * TILEX, self.location[1] * TILEY)

class Door:
	def __init__(self, location, mapnum):
		self.location = location
		self.mapnum = mapnum
		self.target = None
		self.number = -1
	
	def maploc(self):
		return (self.location[1] % maps[self.mapnum].size[1]) * maps[self.mapnum].size[0] + (self.location[0] % maps[self.mapnum].size[0])





