import os

import pygame
from pygame.locals import *

from globals import *
import screen
import font

class Menu:
	def __init__(self):
		self.get_bg()
		self.items = []
		self.selected = 0
		self.cancelable = True
	
	def get_bg(self):
		self.bg = pygame.display.get_surface().copy()
		shade = pygame.surface.Surface((RESX, RESY), 0, self.bg)
		shade.fill((64, 64, 64))
		shade.set_alpha(128)
		self.bg.blit(shade, (0,0))	
	
	def go(self):
		menu_done = False
		
		pygame.display.get_surface().blit(self.bg, (0, 0))
		
		clock = pygame.time.Clock()
		
		while not menu_done:
			for event in pygame.event.get():
				if event.type == KEYDOWN:
					if event.key == Key.quit and self.cancelable:
						menu_done = True
					elif event.key == Key.up:
						selected = (selected - 1) % len(self.items)
					elif event.key == Key.down:
						selected = (selected + 1) % len(self.items)
			
			for item in self.items:
				item.write(False)
			
			pygame.display.flip()
			
			clock.tick(frame_rate)

class MenuItem:
	def __init__(self, text, position):
		self.text = text
		self.position = position
		self.font = font.font
	
	def write(self, active):
		if active:
			color = (64, 64, 255)
		else:
			color = (0, 0, 192)
		# this is to work around a silly bug in certain versions of freetype
		try:
			pygame.display.get_surface().blit(self.font.render(self.text, False, color), self.position)
		except:
			pygame.display.get_surface().blit(self.font.render(self.text, True, color), self.position)

class TownMenu(Menu):
	def __init__(self, town):
		Menu.__init__(self)
		self.items.append(MenuItem("test1", (50, 50))) 
		self.items.append(MenuItem("test2", (50, 60)))








