import pygame

from globals import *
from helper import *
import sprite
import screen
import spritesheet
import status
import map
from screen import scr
from sprite import XSprite
from sprite import XGroup

class Player(XSprite):
	def __init__(self):
		# the sprite sheet here is never used btw
		XSprite.__init__(self, 'P01', (1,2))
		self.torso = spritesheet.spritesheets['PT1']
		self.bottom = spritesheet.spritesheets['PB1']
		self.left = spritesheet.spritesheets['PL1']
		self.right = spritesheet.spritesheets['PR1']
		self.action = self.ac_start
		self.hitbox = pygame.Rect(2, 0, 14, 32)
		self.life = status.maxlife
		self.ammo = status.maxammo
		self.lastdir = 1
		self.framedir = 1
		self.shotdelay = 0
		self.children = XGroup()
		self.life_regen = 0
		self.ammo_regen = 0
		self.framecount = 0
		self.noenter = False
		self.dashtime = 0

	def draw(self, surface):
		if self.lastdir > 0:
			surface.blit(self.left.surf, scr.screen_position(self.location), self.left.frame(self.size, self.frame + self.framedir))
			surface.blit(self.torso.surf, scr.screen_position(self.location), self.torso.frame(self.size, self.frame + self.framedir))
			surface.blit(self.bottom.surf, scr.screen_position(self.location), self.bottom.frame(self.size, self.frame + self.framedir))
			surface.blit(self.right.surf, scr.screen_position(self.location), self.right.frame(self.size, self.frame + self.framedir))
		else:
			surface.blit(self.right.surf, scr.screen_position(self.location), self.right.frame(self.size, self.frame + self.framedir))
			surface.blit(self.torso.surf, scr.screen_position(self.location), self.torso.frame(self.size, self.frame + self.framedir))
			surface.blit(self.bottom.surf, scr.screen_position(self.location), self.bottom.frame(self.size, self.frame + self.framedir))
			surface.blit(self.left.surf, scr.screen_position(self.location), self.left.frame(self.size, self.frame + self.framedir))
			
	def act(self, *params):
		self.framecount = (self.framecount + 1) % 12600
		self.life_regen += 1
		if self.life_regen > status.life_regen_rate:
			self.life_regen = 0
			if self.life < status.maxlife:
				self.life += 1
		self.ammo_regen += 1
		if self.ammo_regen > status.ammo_regen_rate:
			self.ammo_regen = 0
			if self.ammo < status.maxammo:
				self.ammo += 1
		if self.shotdelay > 0:
			self.shotdelay -= 1
		XSprite.act(self, *params)
	
	def ac_start(self, keys):
		if self.can_fall():
			self.action = self.ac_fall
		elif keys[Key.up] and self.can_climb():
			self.action = self.ac_climb
			return
		elif keys[Key.down]:
			self.action = self.ac_crouch
		elif keys[Key.special] and not self.obstructed((self.lastdir,0)) and status.dash:
			self.action = self.ac_dash
		elif keys[Key.jump]:
			self.jumptime = 2
			self.action = self.ac_jump
		elif keys[Key.up]:
			door = scr.map.door_at(self.location)
			if door != None and not self.noenter:
				scr.fade_out()
				scr.map = map.maps[door.target.mapnum]
				#scr.center_on(door.target.location)
				self.location = (door.target.location[0] * TILEX, (door.target.location[1] - 1)* TILEY)
				self.noenter = True
		else:
			self.noenter = False
		
		self.move_on_input(keys)
		if self.vx != 0:
			self.framecount %= 20
			self.frame = 60 + ((self.framecount / 5) * 2)
			if keys[Key.shoot]:
				self.frame += 20
		else:
			self.framecount %= 300
			if keys[Key.shoot]:
				self.frame = 24
			elif self.framecount in range(50, 55) or self.framecount in range(130, 135):
				self.frame = 4
			elif self.framecount in range(260, 295):
				self.frame = 2
			else:
				self.frame = 0
		
		self.shoot_on_input(keys)
	
	def ac_climb(self, keys):
		self.vx = 0
		self.framedir = 0
		self.location = (self.location[0] + 8 - ((self.location[0] + 8) % 16), self.location[1])
		if self.vy < 0:
			self.framecount %= 20
			self.frame = 30 + (self.framecount / 5)
		else:
			self.frame = 30
		
		if keys[Key.jump]:
			self.jumptime = 2
			self.action = self.ac_jump
		elif keys[Key.left] or not self.can_climb():
			self.lastdir = -1
			self.framedir = 0
			self.action = self.ac_fall
		elif keys[Key.right]:
			self.lastdir = 1
			self.framedir = 1
			self.action = self.ac_fall
		elif keys[Key.up]:
			self.vy = -player_speed
		elif keys[Key.down]:
			self.vy = player_speed
		else:
			self.vy = 0
		
	
	def ac_crouch(self, keys):
		self.vx = 0
		self.hitbox = pygame.Rect(2, 6, 14, 26)
		if not keys[Key.down]:
			self.hitbox = pygame.Rect(2, 0, 14, 32)
			self.action = self.ac_start
		
		if keys[Key.special] and status.dodge_roll:
			self.action = self.ac_roll
		elif keys[Key.jump] and status.high_jump:
			self.hitbox = pygame.Rect(2, 0, 14, 32)
			self.vy = -12
			self.jumptime = 3
			self.action = self.ac_hijump
		
		if keys[Key.left]:
			self.lastdir = -1
			self.framedir = 0
		elif keys[Key.right]:
			self.lastdir = 1
			self.framedir = 1
		
		if keys[Key.shoot]:
			self.frame = 8
		else:
			self.frame = 6
		
		self.shoot_on_input(keys, True)
	
	def ac_fall(self, keys):
		if self.vy < 0:
			self.frame = 20
		else:
			self.frame = 22
		
		if (keys[Key.up] or keys[Key.down]) and self.can_climb():
			self.action = self.ac_climb
			return
		
		if not self.can_fall():
			self.action = self.ac_start
		
		if self.vy < TERM_VEL:
			self.vy += GRAVITY
		
		if keys[Key.shoot]:
			self.frame += 20
		
		self.move_on_input(keys)
		self.shoot_on_input(keys)			
		
		if status.wall_grab and ((keys[Key.left] and self.obstructed((-1, 0))) or (keys[Key.right] and self.obstructed((1, 0)))):
			self.action = self.ac_wall_grab
	
	def ac_jump(self, keys):
		self.vy = -8
		self.frame = 20
		self.jumptime -= 1
		if not keys[Key.jump] or self.jumptime <= 0:
			self.action = self.ac_fall
		
		if keys[Key.shoot]:
			self.frame += 20
		
		self.shoot_on_input(keys)
		self.move_on_input(keys)
	
	def ac_hijump(self, keys):
		self.frame = 20
		
		if self.jumptime > 0:
			self.jumptime -= 1
		else:
			self.vy += 1
		
		if self.vy >= 0:
			self.action = self.ac_fall
		
		if keys[Key.shoot]:
			self.frame += 20
		
		self.shoot_on_input(keys)
		self.moveslow_on_input(keys)

	def ac_dash_jump(self, keys):
		self.frame = 20
		
		if self.jumptime > 0:
			self.vy = -6
			self.jumptime -= 1
		else:
			self.vy += GRAVITY / 2
		
		self.vx += -self.lastdir
		
		if self.vx * self.lastdir <= player_speed:
			self.action = self.ac_fall
		
		if self.obstructed((self.lastdir, 0)):
			self.vx = 0
			self.action = self.ac_fall

	def ac_wall_jump(self, keys):
		if self.jumptime == 0:
			self.jumptime = 5
		
		if (keys[Key.left] and self.obstructed((-1, 0))) or (keys[Key.right] and self.obstructed((1, 0))):
			self.action = self.ac_wall_grab
			self.vx = 0
			self.vy = 0
		elif (self.lastdir == -1 and keys[Key.left]) or (self.lastdir == 1 and keys[Key.right]):
			self.frame = 20
			self.vy = -player_speed * 2
			self.vx = self.lastdir * player_speed * 2
		else:
			self.frame = 22
			self.vx = self.lastdir * player_speed * 0.5
			self.vy = 4
		
		if self.jumptime > 0:
			self.jumptime -= 1
		
		if self.jumptime <= 0:
			self.vy = 0
			self.action = self.ac_fall
	
	def ac_wall_grab(self, keys):
		self.framedir = (self.lastdir + 1) / -2
		self.frame = 29
		self.vx = 0
		self.vy = 1
		
		if keys[Key.jump] and status.wall_jump and not ((keys[Key.left] and self.obstructed((-1, 0))) or (keys[Key.right] and self.obstructed((1, 0)))):
			self.lastdir *= -1
			self.framedir = (self.lastdir + 1) / 2
			self.action = self.ac_wall_jump
		
		if not ((keys[Key.left] and self.obstructed((-1, 0))) or (keys[Key.right] and self.obstructed((1, 0)))) and not keys[Key.jump]:
			self.lastdir *= -1
			self.framedir = (self.lastdir + 1) / 2
			self.action = self.ac_fall
		elif not self.can_fall():
			self.lastdir *= -1
			self.framedir = (self.lastdir + 1) / 2
			self.action = self.ac_start
		
		self.shoot_reversed(keys)
	
	def ac_roll(self, keys):
		if self.size != (1, 1):
			self.location = (self.location[0], self.location[1] + 16)
			self.size = (1, 1)
			self.hitbox = pygame.Rect(0, 0, 16, 16)
		
		if (not keys[Key.special] or self.obstructed((self.lastdir * 1.5, 0))) and (not self.obstructed((0, -1))):
			self.framedir = (self.lastdir + 1) / 2
			self.vx = 0
			self.size = (1, 2)
			self.hitbox = pygame.Rect(2, 0, 14, 32)
			self.location = (self.location[0], self.location[1] - 16)
			if keys[Key.down]:
				self.action = self.ac_crouch
			else:
				self.action = self.ac_start
			return
		
		if keys[Key.left]:
			self.lastdir = -1
		elif keys[Key.right]:
			self.lastdir = 1
		
		if self.lastdir == -1:
			self.framedir = 0
			self.frame = 10 + (self.framecount / 2) % 4
			self.vx = player_speed * -1.5
		else:
			self.framedir = 0
			self.lastdir = 1
			self.frame = 30 + (self.framecount / 2)  % 4
			self.vx = player_speed * 1.5
	
	def ac_dash(self, keys):
		if self.dashtime < 1:
			self.vy = player_speed / 2
			self.vx = self.lastdir * player_speed * 4
			self.frame = 26
			self.dashtime = 4
		
		if (self.dashtime > 1):
			self.dashtime -= 1
		elif (self.can_fall()):
			self.vx += - self.lastdir
		else:
			self.vx = self.lastdir * player_speed * 4
		
		if keys[Key.jump] and not self.can_fall():
			self.dashtime = 0
			self.jumptime = 8
			self.action = self.ac_dash_jump
		elif (self.vx * self.lastdir <= player_speed and self.can_fall()) or (self.dashtime <= 1 and not keys[Key.special]) or self.obstructed((self.lastdir, 0)):
			self.dashtime = 0
			self.vx = self.lastdir * player_speed
			self.action = self.ac_start
	
	def ac_ouch(self, keys):
		self.frame = 44 + ((self.framecount % 2) * 2)
		if self.can_fall() and self.vy < TERM_VEL:
			self.vy += GRAVITY
		elif not self.can_fall() and keys[Key.jump]:
			self.vy = -8
		self.move_on_input(keys)
		
		self.vx += self.ouchdir * (player_speed + 1)
		
		if self.framecount >= 4:
			self.action = self.ac_start

	def moveslow_on_input(self, keys):
		global player_speed
		temp = player_speed
		player_speed /= 4
		self.move_on_input(keys)
		player_speed = temp

	def move_on_input(self, keys):
		if keys[Key.left]:
			self.vx = -player_speed
			self.lastdir = -1
			self.framedir = 0
		elif keys[Key.right]:
			self.vx = player_speed
			self.lastdir = 1
			self.framedir = 1
		else:
			self.vx = 0
	
	def shoot_on_input(self, keys, crouch = False):
		if keys[Key.shoot] and self.shotdelay <= 0 and self.ammo > 0:
			self.ammo -= 1
			self.shotdelay = status.shot_timeout
			shot = sprite.Shot((self.lastdir * 6, 0))
			if not crouch:
				shot.location = sum_tuple(self.location, (12 * self.lastdir, 8))
			else:
				shot.location = sum_tuple(self.location, (12 * self.lastdir, 15))
			self.children.add(shot)
			return True
		return False

	def shoot_reversed(self, keys, crouch = False):
		if keys[Key.shoot] and self.shotdelay <= 0 and self.ammo > 0:
			self.ammo -= 1
			self.shotdelay = status.shot_timeout
			shot = sprite.Shot((self.lastdir * -6, 0))
			if not crouch:
				shot.location = sum_tuple(self.location, (-12 * self.lastdir, 8))
			else:
				shot.location = sum_tuple(self.location, (-12 * self.lastdir, 15))
			self.children.add(shot)
			return True
		return False

	def hit(self, enemy):
		self.life -= enemy.damage
		if enemy.get_rectangle().left > self.get_rectangle().left:
			self.ouchdir = -1
		else:
			self.ouchdir = 1
		
		if self.action == self.ac_ouch:
			self.framecount -= 2
		else:
			self.framecount = 0
			self.frame = 44
			self.action = self.ac_ouch